@@ -9,17 +9,17 @@
  * Copyright (c) 2006-2007 Logitech.
  *
  * This file is part of libwebcam.
- * 
+ *
  * libwebcam is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published
  * by the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
- * 
+ *
  * libwebcam is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public License
  * along with libwebcam.  If not, see <http://www.gnu.org/licenses/>.
  */
@@ -27,7 +27,9 @@
 #ifndef C_WEBCAM_H
 #define C_WEBCAM_H
 
-
+#include <inttypes.h>
+#include <sys/types.h>
+#include "dynctrl-logitech.h"
 /*
  * Basic type definitions
  */
@@ -57,6 +59,7 @@ enum _CResults {
 	C_NOT_FOUND,			///< Object not found
 	C_BUFFER_TOO_SMALL,		///< Buffer too small
 	C_SYNC_ERROR,			///< Error during data synchronization
+	// 10:
 	C_NO_MEMORY,			///< Out of memory
 	C_NO_HANDLES,			///< Out of handles
 	C_V4L2_ERROR,			///< A Video4Linux2 API call returned an unexpected error
@@ -64,6 +67,13 @@ enum _CResults {
 	C_PARSE_ERROR,			///< A control could not be parsed
 	C_CANNOT_WRITE,			///< Writing not possible (e.g. read-only control)
 	C_CANNOT_READ,			///< Reading not possible (e.g. write-only control)
+	C_INVALID_XU_CONTROL,	///< Invalid XU control has been encountered
+	C_SYSTEM_ERROR,			///< An system call returned an unexpected error
+	C_DEVICE_BUSY,			///< The device is busy
+	// 20:
+	C_WRONG_STATE,			///< The requested operation is invalid at this point
+	C_INVALID_FORMAT,		///< The requested format is not available or not supported
+	C_CONVERSION_ERROR,		///< The format conversion failed (invalid format, buffer size, etc.)
 };
 
 
@@ -105,6 +115,8 @@ typedef enum _CControlId {
 	CC_EXPOSURE_TIME_ABSOLUTE,
 	/// Relative change to the current length of exposure
 	CC_EXPOSURE_TIME_RELATIVE,
+	/// Auto brightness
+	CC_AUTO_BRIGHTNESS,
 
 	// UVC: Optics control
 	/// Enable/disable automatic focus adjustment
@@ -176,7 +188,15 @@ typedef enum _CControlId {
 	CC_LOGITECH_DISABLE_PROCESSING,
 	// Bits per pixel for raw (Bayer) mode
 	CC_LOGITECH_RAW_BITS_PER_PIXEL,
+	//
+	CC_LOGITECH_USB_VID,
+	//
+	CC_LOGITECH_USB_PID,
+	//
+	CC_LOGITECH_USB_BCD,
 
+	/// Base for raw UVC controls
+	CC_UVC_XU_BASE	= 0xFFFF0000,
 
 } CControlId;
 
@@ -335,7 +355,7 @@ typedef struct _CDevice {
 	/// a device appears as '/dev/video0', the short name is 'video0'.
 	/// This member is never NULL.
 	char			* shortName;
-	
+
 	/// The human-readable name of the device.
 	/// This member is never NULL.
 	char			* name;
@@ -361,8 +381,14 @@ typedef struct _CDevice {
  */
 typedef struct _CControlRawValue {
 	/// Pointer to the raw data.
+	/// Note: Callers must allocate and free this buffer by themselves.
 	void			* data;
 	/// Size of the raw data.
+	/// For controls of type #CC_TYPE_RAW this size must be at least as big as
+	/// the CControl.max.raw.size value of the control in question.
+	/// For controls of type #CC_TYPE_STRING this size minus one must be between the
+	/// the CControl.string.min_length and CControl.string.max_length values
+	/// and must be a multiple of the CControl.string.length_step value.
 	unsigned int	size;
 
 } CControlRawValue;
@@ -387,6 +413,10 @@ typedef struct _CControlValue {
 		/// The value of the control for raw cntrols.
 		/// This member is valid only for control type #CC_TYPE_RAW.
 		CControlRawValue	raw;
+
+		/// The value of the control for string controls.
+		/// This member is valid only for control type #CC_TYPE_STRING.
+		CControlRawValue	string;
 	};
 
 } CControlValue;
@@ -448,7 +478,7 @@ typedef struct _CControl {
 			unsigned int	count;
 			/// The list of choice values
 			CControlChoice	* list;
-			
+
 			/// A buffer that contains the names of the #CControlChoice items.
 			/// The @a name fields of the items point to strings within this buffer,
 			/// so there is no need to work with this member directly except
@@ -456,6 +486,19 @@ typedef struct _CControl {
 			//char			* names;
 
 		} choices;
+
+		/// Attributes for string controls.
+		/// The members of this struct are only valid if the type is #CC_TYPE_STRING.
+		/// Note that the length values do not include the terminating null byte.
+		struct {
+			/// The minimum number of characters required
+			unsigned int	min_length;
+			/// The maximum number of characters allowed
+			unsigned int	max_length;
+			/// The value of which the string length has to be a multiple
+			unsigned int	length_step;
+
+		} string;
 	};
 
 } CControl;
@@ -487,7 +530,7 @@ typedef struct _CPixelFormat {
 typedef struct _CFrameSize {
 	/// Type of the frame size(s) described (discrete, continuous, step-wise)
 	CFrameSizeTypes	type;
-	
+
 	union {
 		/// Discrete frame size.
 		/// This member is only valid if @a type is #CF_SIZE_DISCRETE.
@@ -716,6 +759,8 @@ extern CResult		c_unsubscribe_event (CHandle hDevice, CEventId event_id);
 #ifndef DISABLE_UVCVIDEO_DYNCTRL
 extern CResult		c_add_control_mappings_from_file (const char *file_name, CDynctrlInfo *info);
 extern CResult		c_add_control_mappings (CHandle handle, const char *file_name, CDynctrlInfo *info);
+extern CResult 		c_read_xu_control(CHandle hDevice, unsigned char entity[], uint16_t unit_id, unsigned char selector, CControlValue *value);
+extern CResult 		c_write_xu_control(CHandle hDevice, unsigned char entity[], uint16_t unit_id, unsigned char selector, CControlValue *value);
 #endif
 
 extern char			*c_get_error_text (CResult error);
