@@ -22,6 +22,8 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <inttypes.h>
+#include <sys/types.h>
 #include <string.h>
 #include <dirent.h>
 #include <fnmatch.h>
@@ -29,7 +31,6 @@
 #include <assert.h>
 #include <unistd.h>
 #include <errno.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
@@ -788,7 +789,6 @@ add_control_mappings(CHandle hDevice, const char *filename)
 	return res;
 }
 
-
 int
 main (int argc, char **argv)
 {
@@ -945,6 +945,38 @@ main (int argc, char **argv)
 		}
 		printf("%d\n", value.value);
 	}
+	// Retrieve raw control value
+	else if(args_info.get_raw_given) {
+		//scan input
+		uint16_t unit_id;
+		unsigned char selector;
+		sscanf(args_info.get_raw_arg, "%hu:%hhu", &unit_id, &selector);
+		CControlValue value;
+		value.type = CC_TYPE_RAW;
+		// the entity is only used for the generating a control name
+		//TODO: pass the guid through cmdline (optional)
+		unsigned char entity[] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
+		res = c_read_xu_control(handle, entity, unit_id, selector, &value);
+		if(res) {
+			print_handle_error(handle, "Unable to retrieve control value", res);
+			goto done;
+		}
+		
+		//print the raw value
+		uint8_t * val = value.raw.data;
+		int i=0;
+		printf("query current value of: (LE)0x");
+		for(i=0; i<value.raw.size; i++)	{
+			printf("%.2x", val[i]);
+		}
+		printf("  (BE)0x");
+		for(i=value.raw.size-1; i >=0; i--) {
+			printf("%.2x", val[i]);
+		}
+		printf("\n");
+		//free the raw value alocation
+		if(value.raw.data) free(value.raw.data);
+	}
 	else if(args_info.set_given) {
 		CControlValue value;
 
@@ -975,6 +1007,47 @@ main (int argc, char **argv)
 			goto done;
 		}
 	}
+	// Set the raw control value
+	else if(args_info.set_raw_given) {
+		CControlValue value;
+		
+		// Parse the control value
+		if(args_info.inputs_num < 1) {
+			res = 3;
+			print_error("No raw control value specified", -1);
+			goto done;
+		}
+		uint16_t unit_id;
+		unsigned char selector;
+
+		sscanf(args_info.set_raw_arg, "%hu:%hhu", &unit_id, &selector);
+		
+		parse_raw_control_value (args_info.inputs[0], &value);
+		
+		// the entity is only used for the generating a control name
+		//TODO: pass the guid through cmdline (optional)
+		unsigned char entity[] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
+		res = c_write_xu_control(handle, entity, unit_id, selector, &value);
+		if(res) {
+			print_handle_error(handle, "Unable to set the control value", res);
+			goto done;
+		}
+		
+		//print the raw value le and be format
+		uint8_t * val = value.raw.data;
+		int i=0;
+		printf("set value of          : (LE)0x");
+		for(i=0; i<value.raw.size; i++)	{
+			printf("%.2x", val[i]);
+		}
+		printf("  (BE)0x");
+		for(i=value.raw.size-1; i >=0; i--) {
+			printf("%.2x", val[i]);
+		}
+		printf("\n");
+		//free the raw value alocation
+		if(value.raw.data) free(value.raw.data);
+	}
 	else if(args_info.save_ctrl_given) {
 		res = save_controls( handle, args_info.save_ctrl_arg);
 	}
